@@ -13,12 +13,14 @@ from dask_cloudprovider.aws.helper import (
     get_default_vpc,
     get_vpc_subnets,
     get_security_group,
+    dict_to_aws,
 )
 from dask_cloudprovider.utils.timeout import Timeout
 
 try:
     from aiobotocore.session import get_session
     import botocore.exceptions
+    import botocore.config
 except ImportError as e:
     msg = (
         "Dask Cloud Provider AWS requirements are not installed.\n\n"
@@ -50,6 +52,9 @@ class EC2Instance(VMInterface):
         filesystem_size=None,
         key_name=None,
         iam_instance_profile=None,
+        instance_tags: None,
+        volume_tags: None,
+        use_private_ip: False,
         **kwargs,
     ):
         super().__init__(*args, **kwargs)
@@ -71,6 +76,9 @@ class EC2Instance(VMInterface):
         self.filesystem_size = filesystem_size
         self.key_name = key_name
         self.iam_instance_profile = iam_instance_profile
+        self.instance_tags = instance_tags
+        self.volume_tags = volume_tags
+        self.use_private_ip = use_private_ip
 
     async def create_vm(self):
         """
@@ -79,8 +87,9 @@ class EC2Instance(VMInterface):
         """
         # TODO Enable Spot support
 
+        boto_config = botocore.config.Config(retries=dict(max_attempts=10))
         async with self.cluster.boto_session.create_client(
-            "ec2", region_name=self.region
+            "ec2", region_name=self.region, config=boto_config
         ) as client:
             self.vpc = self.vpc or await get_default_vpc(client)
             self.subnet_id = (
@@ -117,14 +126,26 @@ class EC2Instance(VMInterface):
                 "InstanceInitiatedShutdownBehavior": "terminate",
                 "NetworkInterfaces": [
                     {
-                        "AssociatePublicIpAddress": True,
+                        "AssociatePublicIpAddress": False
+                        if self.use_private_ip
+                        else True,
                         "DeleteOnTermination": True,
-                        "Description": "public",
+                        "Description": "private" if self.use_private_ip else "public",
                         "DeviceIndex": 0,
                         "Groups": self.security_groups,
                         "SubnetId": self.subnet_id,
                     }
                 ],
+                "TagSpecifications": [
+                    {
+                        "ResourceType": "instance",
+                        "Tags": dict_to_aws(self.instance_tags, upper=True),
+                    },
+                    {
+                        "ResourceType": "volume",
+                        "Tags": dict_to_aws(self.volume_tags, upper=True),
+                    },
+                ],
             }
 
             if self.key_name:
@@ -140,43 +161,66 @@ class EC2Instance(VMInterface):
 
             response = await client.run_instances(**vm_kwargs)
             [self.instance] = response["Instances"]
-            await client.create_tags(
-                Resources=[self.instance["InstanceId"]],
-                Tags=[
-                    {"Key": "Name", "Value": self.name},
-                    {"Key": "Dask Cluster", "Value": self.cluster.uuid},
-                ],
-            )
-            self.cluster._log(
-                f"Created instance {self.instance['InstanceId']} as {self.name}"
-            )
 
-            timeout = Timeout(
-                300,
-                f"Failed Public IP for instance {self.instance['InstanceId']}",
-            )
-            while (
-                "PublicIpAddress" not in self.instance
-                or self.instance["PublicIpAddress"] is None
-            ) and timeout.run():
-                backoff = 0.1
-                await asyncio.sleep(
-                    min(backoff, 10) + backoff % 1
-                )  # Exponential backoff with a cap of 10 seconds and some jitter
-                try:
-                    response = await client.describe_instances(
-                        InstanceIds=[self.instance["InstanceId"]], DryRun=False
-                    )
-                    [reservation] = response["Reservations"]
-                    [self.instance] = reservation["Instances"]
-                except botocore.exceptions.ClientError as e:
-                    timeout.set_exception(e)
-                backoff = backoff * 2
-            return self.instance["PublicIpAddress"]
+            try:  # Ensure we tear down any resources we allocated if something goes wrong
+                return await self.configure_vm(client)
+            except Exception:
+                self.cluster._log(
+                    f"reclaiming vm because configure_vm failed {self.name}"
+                )
+                await self.destroy_vm()
+                raise
+
+    async def configure_vm(self, client):
+        timeout = Timeout(300, f"Failed to add tags for {self.instance['InstanceId']}")
+        backoff = 0.1
+        while timeout.run():
+            try:
+                await client.create_tags(
+                    Resources=[self.instance["InstanceId"]],
+                    Tags=[
+                        {"Key": "Name", "Value": self.name},
+                        {"Key": "Dask Cluster", "Value": self.cluster.uuid},
+                    ],
+                )
+                break
+            except Exception as e:
+                timeout.set_exception(e)
+
+            await asyncio.sleep(min(backoff, 10) + backoff % 1)
+            # Exponential backoff with a cap of 10 seconds and some jitter
+            backoff = backoff * 2
+
+        self.cluster._log(
+            f"Created instance {self.instance['InstanceId']} as {self.name}"
+        )
+
+        address_type = "Private" if self.use_private_ip else "Public"
+        ip_address_key = f"{address_type}IpAddress"
+
+        default_error = (
+            f"Failed {address_type} IP for instance {self.instance['InstanceId']}"
+        )
+        timeout = Timeout(300, default_error)
+        backoff = 0.1
+        while self.instance.get(ip_address_key) is None and timeout.run():
+            try:
+                response = await client.describe_instances(
+                    InstanceIds=[self.instance["InstanceId"]], DryRun=False
+                )
+                [reservation] = response["Reservations"]
+                [self.instance] = reservation["Instances"]
+            except botocore.exceptions.ClientError as e:
+                timeout.set_exception(e)
+            await asyncio.sleep(min(backoff, 10) + backoff % 1)
+            # Exponential backoff with a cap of 10 seconds and some jitter
+            backoff = backoff * 2
+        return self.instance[ip_address_key]
 
     async def destroy_vm(self):
+        boto_config = botocore.config.Config(retries=dict(max_attempts=10))
         async with self.cluster.boto_session.create_client(
-            "ec2", region_name=self.region
+            "ec2", region_name=self.region, config=boto_config
         ) as client:
             await client.terminate_instances(
                 InstanceIds=[self.instance["InstanceId"]], DryRun=False
@@ -302,6 +346,16 @@ class EC2Cluster(VMCluster):
         be created automatically. Default is ``True``.
     debug: bool, optional
         More information will be printed when constructing clusters to enable debugging.
+    instance_tags: dict, optional
+        Tags to be applied to all EC2 instances upon creation. By default, includes
+        "createdBy": "dask-cloudprovider"
+    volume_tags: dict, optional
+        Tags to be applied to all EBS volumes upon creation. By default, includes
+        "createdBy": "dask-cloudprovider"
+    use_private_ip: bool (optional)
+        Whether to use a private IP (if True) or public IP (if False).
+
+        Default ``False``.
 
     Notes
     -----
@@ -407,6 +461,9 @@ class EC2Cluster(VMCluster):
         iam_instance_profile=None,
         docker_image=None,
         debug=False,
+        instance_tags=None,
+        volume_tags=None,
+        use_private_ip=False,
         **kwargs,
     ):
         self.boto_session = get_session()
@@ -458,6 +515,15 @@ class EC2Cluster(VMCluster):
             else self.config.get("iam_instance_profile")
         )
         self.debug = debug
+
+        instance_tags = instance_tags if instance_tags is not None else {}
+        self.instance_tags = {**instance_tags, **self.config.get("instance_tags")}
+
+        volume_tags = volume_tags if volume_tags is not None else {}
+        self.volume_tags = {**volume_tags, **self.config.get("volume_tags")}
+
+        self._use_private_ip = use_private_ip
+
         self.options = {
             "cluster": self,
             "config": self.config,
@@ -474,6 +540,9 @@ class EC2Cluster(VMCluster):
             "filesystem_size": self.filesystem_size,
             "key_name": self.key_name,
             "iam_instance_profile": self.iam_instance_profile,
+            "instance_tags": self.instance_tags,
+            "volume_tags": self.volume_tags,
+            "use_private_ip": self._use_private_ip,
         }
         self.scheduler_options = {**self.options}
         self.worker_options = {**self.options}
