@@ -1004,6 +1004,13 @@ class ECSCluster(SpecCluster):
             class SchedulerAddress(object):
                 def __init__(self_):
                     self_.address = self._scheduler_address
+                    self_.status = Status.running
+
+                def __await__(self):
+                    async def _():
+                        return self
+
+                    return _().__await__()
 
             self.scheduler = SchedulerAddress()
 
