@@ -72,6 +72,7 @@ class SchedulerMixin(object):
     ):
         super().__init__(*args, **kwargs)
         self.name = f"dask-{self.cluster.uuid}-scheduler"
+        self.port = scheduler_options.get("port", 8786)
         self.command = " ".join(
             [
                 self.set_env,
@@ -85,7 +86,7 @@ class SchedulerMixin(object):
     async def start(self):
         self.cluster._log("Creating scheduler instance")
         ip = await self.create_vm()
-        self.address = f"{self.cluster.protocol}://{ip}:8786"
+        self.address = f"{self.cluster.protocol}://{ip}:{self.port}"
         await self.wait_for_scheduler()
         await super().start()
 
@@ -280,6 +281,7 @@ class VMCluster(SpecCluster):
         self.scheduler_options["env_vars"] = env_vars
         self.scheduler_options["protocol"] = protocol
         self.scheduler_options["scheduler_options"] = scheduler_options
+        self.scheduler_options["extra_bootstrap"] = extra_bootstrap
         self.worker_options["env_vars"] = env_vars
         self.options["docker_args"] = docker_args
         self.options["extra_bootstrap"] = extra_bootstrap
@@ -289,6 +291,7 @@ class VMCluster(SpecCluster):
         self.worker_options["worker_class"] = worker_class
         self.worker_options["protocol"] = protocol
         self.worker_options["worker_options"] = worker_options
+        self.worker_options["extra_bootstrap"] = extra_bootstrap
         self.uuid = str(uuid.uuid4())[:8]
 
         super().__init__(**kwargs, security=self.security)
